package de.historischekommission_muenchen.oxydition.common.dialogs;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import de.historischekommission_muenchen.oxydition.common.dialogs.elements.OxyditionInputFilter;

/**
 * Modulare Dialogfenstervorlage, für Operationen mit Benutzereingabe. Die
 * Vorlage ist darauf ausgelegt, durch erbende Klassen in Aufbau und Funktion an
 * die jeweiligen Anforderungen angepasst zu werden. Hierfür können nach Bedarf
 * Elemente wie Textfelder, TextAreas oder Checkboxes zum Dialogfenster
 * hinzugefügt werden. Ebenso kann das Verhalten der Funktionsabschlussbuttons
 * angepasst werden.
 * 
 * @author Maximilian Schrott
 */
public class ModularGridBagDialog extends OxyditionOperationDialog {

	private static final long serialVersionUID = 3321298428588583660L;

	// Variablen
	private JPanel inputPanel;
	private GridBagConstraints cstr; // Layout für das Fenster

	// Konstruktor
	public ModularGridBagDialog(JFrame owner, String title) {
		super(owner, title);
		constructDialogBaseLayout();
	}

	public ModularGridBagDialog(JFrame owner) {
		super(owner, "Dialogfenster");
		constructDialogBaseLayout();
	}

	private void constructDialogBaseLayout() {

		// Variablen initialisieren
		inputPanel = new JPanel(new GridBagLayout());
		initializeGridBagConstraints();
		mainContent.add(inputPanel, BorderLayout.CENTER);

	}

	private void initializeGridBagConstraints() {
		cstr = new GridBagConstraints();
		cstr.gridx = 0;
		cstr.gridy = -1;
		cstr.gridwidth = 1;
		cstr.fill = GridBagConstraints.HORIZONTAL;
		cstr.insets = new Insets(1, 0, 1, 0);
	}

	/**
	 * Fügt ein Texteingabefeld mit Beschriftung zum Dialogfenster hinzu und gibt es
	 * zurück
	 * 
	 * @param labeltext - Beschriftung für das Textfeld
	 * @return das erzeugte Textfeld
	 */
	protected JTextField addField(String labeltext) {

		gridBagNewLine();
		inputPanel.add(new JLabel(labeltext + ": "), cstr);

		cstr.gridx = 1;
		cstr.weightx = 1;
		JTextField textField = new JTextField();
		textField.setColumns(25);
		OxyditionInputFilter.setForInput(textField);

		inputPanel.add(textField, cstr);

		return textField;
	}

	/**
	 * Fügt ein leeres Panel als vertikalen Platzhalter zum Dialogfenster hinzu
	 */
	protected void addEmptySpace() {

		gridBagNewLine();

		JPanel emptySpace = new JPanel();
		emptySpace.setMinimumSize(new Dimension(1, 4));
		inputPanel.add(emptySpace, cstr);

	}

	/**
	 * Fügt eine Checkbox mit Beschriftung zum Dialogfenster hinzu und gibt die
	 * Checkbox zurück
	 * 
	 * @param labeltext - Beschriftung für die Checkbox
	 * @return die erstellte Checkbox
	 */
	protected JCheckBox addCheckBox(String labeltext) {

		gridBagNewLine();

		JCheckBox checkBox = new JCheckBox(labeltext);
		inputPanel.add(checkBox, cstr);
		return checkBox;

	}

	/**
	 * Fügt eine TextArea zum Dialogfenster hinzu, dass über die komplette Breite
	 * geht
	 * 
	 * @param text - Der Text der in der TextArea angezeigt werden soll
	 * @return die erstellte TextArea
	 */
	protected JTextArea addFullWidthTextArea(String text) {

		gridBagNewLine();

		cstr.gridwidth = 2;
		JTextArea textarea = new JTextArea(text);
		inputPanel.add(textarea, cstr);
		cstr.gridwidth = 1;
		OxyditionInputFilter.setForInput(textarea);

		return textarea;
	}

	/**
	 * Fügt eine nicht edierbare TextArea für Hilfetexte hinzu, das über die
	 * komplette Breite geht
	 * 
	 * @param text - Der Hilfstext, der in der TextArea angezeigt werden soll
	 * @return die erstellte TextArea
	 */
	protected JTextArea addFullWithHelpTextArea(String text) {

		gridBagNewLine();

		cstr.gridwidth = 2;
		JTextArea textArea = new JTextArea(text);
		inputPanel.add(textArea, cstr);
		textArea.setEditable(false);
		cstr.gridwidth = 1;

		return textArea;

	}

	protected void addDoubleTextLabel(String leftText, String rightText) {

		gridBagNewLine();

		inputPanel.add(new JLabel(leftText + ": "), cstr);

		cstr.gridx = 1;
		cstr.weightx = 1;
		inputPanel.add(new JLabel(rightText), cstr);

	}

	protected JComboBox<String> addComboBox(String labeltext, String[] selection) {

		gridBagNewLine();
		inputPanel.add(new JLabel(labeltext + ": "), cstr);

		cstr.gridx = 1;
		cstr.weightx = 1;
		JComboBox<String> comboBox = new JComboBox<String>(selection);
		inputPanel.add(comboBox, cstr);

		return comboBox;

	}

	private void gridBagNewLine() {

		cstr.gridy += 1;
		cstr.gridx = 0;
		cstr.weightx = 0;
	}

	/**
	 * Fügt zwei Buttons zum Bestätigen oder Abbrechen der Funktion in das
	 * Dialogfenster ein
	 * 
	 * @param okText     - Der Buttontext für den Bestätigen-Button
	 * @param cancelText - Der Buttontext für den Abbrechen-Button
	 */
	protected void insertOperationEndButtonPanel(String okText, String cancelText) {

		operationEndButtonPanel = assembleOperationEndButtonPanel("Übernehmen", "Abbrechen");
		mainContent.add(operationEndButtonPanel, BorderLayout.PAGE_END);
	}

	/**
	 * Stellt das Dialogfenster zusammen und macht es sichtbar
	 */
	protected void readyDialog() {

		this.getContentPane().add(mainContent);
		this.pack();
		this.setLocationRelativeTo(this.getOwner());
		this.setVisible(true);
	}

	@Override
	protected void cancelButtonAction() {
		dispose();
	}
}
