package de.historischekommission_muenchen.oxydition.common.dialogs.elements;

import java.util.ArrayList;

import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.ListSelectionModel;

/**
 * JList-Spezialisierung, die neben dem normalen, sichtbaren Listeninhalt noch
 * eine zweite, versteckte Liste verwaltet. Die zweite Liste enthält
 * Metainformationen zu den Einträgen aus der ersten Liste und wird mit dieser
 * synchron gehalten. Die erste Liste kann mittels der Methode updateJList()
 * gefiltert werden. Auf die sichtbare Liste kann mit den Methoden aus JList
 * zugegriffen werden, auf die versteckte mit Methoden aus dieser Klasse.
 * 
 * @param <E>
 *            - Objekttyp, der von der Liste verwaltet werden soll (auf String
 *            ausgerichtet)
 */
public class UpdatingJList<E> extends JList<E> {

	private static final long serialVersionUID = 4043472303348193843L;

	//Zweite Liste für Metainformationen
	private ArrayList<String> metaDataList;

	// Kopien von Eintrags- und Meta-Liste zur Wiederherstellung des Originalzustandes
	private String[] displayDataBackup;
	private String[] metaDataBackup;

	/**
	 * Konstruktor
	 * 
	 * @param displayData
	 *            Array der Werte, die in der Liste dargestellt
	 *            werden sollen
	 * @param metaData
	 *            Array der zu den dargestellten Einträgen gehörenden Metainformationen
	 */

	public UpdatingJList(String[] displayData, String[] metaData) {

		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		fillList(displayData, metaData);
	}

	@SuppressWarnings("unchecked")
	public void fillList(String[] displayData, String[] metaData) {
		DefaultListModel<String> displayDataModel = new DefaultListModel<String>();
		displayDataBackup = new String[displayData.length];
		metaDataList = new ArrayList<String>();
		metaDataBackup = metaData;

		// Anzeigeeinträge und Meta-Werte synchron in Listen
		// eintragen
		if (displayData.length == metaData.length) {
			for (int i = 0; i < displayData.length; i++) {

				String displayEntry = cleanDisplayEntry(displayData[i]);

				displayDataModel.add(i, displayEntry);
				displayDataBackup[i] = displayEntry;
				metaDataList.add(i, metaData[i]);
			}
		} else {
			// Fehler werfen, falls Anzeige- und Meta-Werte-Listen nicht gleich
			// lang sind, da dies die korrekte Funktion der Liste unmöglich macht
			throw new Error("Asynchronous Data Lists");
		}

		setModel((ListModel<E>) displayDataModel);

	}

	private String cleanDisplayEntry(String displayEntry) {

		String entry = displayEntry;

		entry = entry.trim();
		entry = entry.replaceAll("\r\n", " ");
		entry = entry.replaceAll("\r", " ");
		entry = entry.replaceAll("\t", " ");
		entry = entry.replaceAll("\n", " ");
		entry = entry.replaceAll("\\s{2,}", " ");

		return entry;

	}

	/**
	 * Reduziert den Inhalt der Liste auf Einträge (und Metadaten), deren Wert mit dem
	 * übergebenen Wert ganz oder teilweise übereinstimmen
	 * 
	 * @param newInput
	 *            String nachdem die Liste gefiltert werden soll
	 */
	@SuppressWarnings("unchecked")
	public void updateList(String newInput) {	
		newInput = newInput.toLowerCase();
		DefaultListModel<String> updatedListModel = new DefaultListModel<String>();
		metaDataList.clear();
		
		for (int i = 0; i < displayDataBackup.length; i++) {
			
			if (displayDataBackup[i].toLowerCase().contains(newInput)) {
				updatedListModel.addElement(displayDataBackup[i]);
				metaDataList.add(metaDataBackup[i]);				
			}			
		}	

		this.setModel((ListModel<E>) updatedListModel);
		this.setSelectedIndex(0);
	}
	
	/**
	 * Gibt den Metawert des markierten Listeneintrags zurück
	 */
	public String getSelectedMetaEntry() {
		return getMetaEntryAt(getSelectedIndex());
	}

	/**
	 * Gibt den Metawert des Eintrags an der übergebenen Indexposition zurück
	 * 
	 * @param index
	 *            Der Index des zurückzugebenden Elements
	 * @return Den Metadatenwert an der Indexposition
	 */
	public String getMetaEntryAt(int index) {
		return metaDataList.get(index);
	}

}
