package de.historischekommission_muenchen.oxydition.common.operations;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.beans.AutoTagBean;
import de.historischekommission_muenchen.oxydition.common.beans.TagReferenceBookmarkBean;
import de.historischekommission_muenchen.oxydition.common.dialogs.AutoTagDialog;
import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.StandardReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation, mit der die Benutzer*innen halbautomatisch bestimmte
 * Textstellen im Dokument zu finden und zu referenzieren können. Die
 * Benutzer*innen markieren einen Textausschnitt im Dokument und lösen die
 * Operation aus. Darauf wird ein AutoTagDialog-Fenster geöffnet, das das
 * Dokument nach allen Vorkommen des markierten Textes durchsucht und den
 * Benutzer*innen bei jedem Suchtreffer die Wahl lässt mittels eines <rs>-Tags
 * mit der Registerdatenbank zu verknüpfen. Die Operationsargumente legen dabei
 * fest auf welche RDB-Datei verwiesen wird und welche Typen von Verweisen
 * möglich sind ("bis zu drei").
 * 
 * @author Maximilian Schrott
 *
 */
public class AutoTagOperation extends ReadRegisterOperation {

	/*
	 * OPERATIONSSPEZIFISCHE ARGUMENTE EINRICHTEN
	 */
	private static String ARGUMENT_TYPEONE = "Erster Type";
	private static String ARGUMENT_TYPETWO = "Zweiter Type (optional)";
	private static String ARGUMENT_TYPETHREE = "Dritter Type (optional)";
	private static String ARGUMENT_GENERALTYPE = "Übergreifender Type (optional)";

	private static String typeOneDescription = "Der (erste) Wert der dem type-Attribut des einzufügenden <rs>-Elements gegeben werden kann.\n"
			+ "Sind mehr als ein Type-Argument angegeben, dann erhalten die Bearbeiter*innen bei jeder mit dieser Operation"
			+ " gefundenen Textstelle die Wahl, mit welchem Attribut sie sie referenzieren möchten.";
	private static String typeTwoDescription = "Der zweite mögliche Wert für das type-Attribut des einzufügenden <rs>-Elements. "
			+ "WICHTIG: Feld leer lassen, wenn für die Operation kein zweites Argument benötigt wird.";
	private static String typeThreeDescription = "Der dritte mögliche Wert für das type-Attribut des einzufügenden <rs>-Elements. "
			+ "WICHTIG: Feld leer lassen, wenn dür die Operation kein drittes Argument benötigt wird.";
	private static String generalTypeDescription = "Ein allgemeiner Bezeichner für alle Types die mit dieser Operation "
			+ "markiert werden sollen. WICHTIG: Dient nur zu Anzeigezwecken. "
			+ "Wird dieses Feld leergelassen wird der Wert von 'Erster Type' als übergreifender Type benutzt.";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {
			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription),
			new ArgumentDescriptor(ARGUMENT_NODE, ArgumentDescriptor.TYPE_STRING, nodeArgDescription),
			new ArgumentDescriptor(ARGUMENT_EXPRESSION, ArgumentDescriptor.TYPE_STRING, expressionArgDescription),

			new ArgumentDescriptor(ARGUMENT_TYPEONE, ArgumentDescriptor.TYPE_STRING, typeOneDescription),
			new ArgumentDescriptor(ARGUMENT_TYPETWO, ArgumentDescriptor.TYPE_STRING, typeTwoDescription),
			new ArgumentDescriptor(ARGUMENT_TYPETHREE, ArgumentDescriptor.TYPE_STRING, typeThreeDescription),
			new ArgumentDescriptor(ARGUMENT_GENERALTYPE, ArgumentDescriptor.TYPE_STRING, generalTypeDescription) };

	@Override
	public String getDescription() {

		return "Die Benutzer*innen markieren einen Abschnitt des Texts und rufen diese Operation aus."
				+ " Entsprechend der Argumente greift die Operation auf eine Registerdatenbank zurück und zeigt bestimmte"
				+ " Einträge daraus als Vorschläge für die Benutzer*innen an. Gleichzeitig wird das Dokument nach allen"
				+ " Vorkommnissen des markierten Textausschnitts durchsucht. Die Benutzer*innen wählen einen der vorgeschlagenen Einträge aus"
				+ " der Registerdatenbank aus und entscheideen bei jedem Suchtreffer, ob es sich tatsächlich um ein"
				+ " Vorkommen des gesuchten Begriffs oder einen flaschen Treffer handelt. In letzterem Fall wird"
				+ " der Treffer übersprungen. Andernfalls können die Benutzer*innen die Textstelle für die Referenzierung vormerken."
				+ " Dabei haben sie, je nach Anzahl der ausgefüllten Typ-Argumente die Auswahl zwischen"
				+ " einer und drei Optionen, die über den Inhalt des type-Attributs der Referenz bestimmen."
				+ " Beim erfolgreichen Abschluss der Operation werden dann alle vorgemerkten Treffer mit <rs>-Elementen umgeben,"
				+ " die auf den ausgewählten Eintrag in der voreingestellten Registerdatenbanksdatei verweisen.";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		String selectedText = editorAccess.getSelectedText();
		selectedText = selectedText.trim();

		if (selectedText.isEmpty()) {
			displayNoSelectionMessage();
			return;
		}

		if (!validateArguments(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			ReadRegisterBasic registerReader = getRegisterEntries(args);
			String[] eintraege = registerReader.getEintrag();
			String[] ids = registerReader.getID();
			Object[] typeArguments = collectTypeArguments(args);

			AutoTagDialog dlg = new AutoTagDialog(oxygenFrame, eintraege, ids, authorAccess, typeArguments,
					selectedText);

			AutoTagBean input = dlg.displayDialog();
			if (input.isFilled) {
				executeReferencing(input);
			}

		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}

	}

	protected boolean validateArguments(ArgumentsMap args) {

		boolean allValid = true;

		allValid = validateBaseArguments(args);

		if (!OperationUtility.isValidArgument(args.getArgumentValue(ARGUMENT_TYPEONE))) {
			allValid = false;
		}

		return allValid;
	}

	protected Object[] collectTypeArguments(ArgumentsMap args) {

		Object[] typeArguments = new Object[4];

		typeArguments[0] = args.getArgumentValue(ARGUMENT_GENERALTYPE);
		typeArguments[1] = args.getArgumentValue(ARGUMENT_TYPEONE);
		typeArguments[2] = args.getArgumentValue(ARGUMENT_TYPETWO);
		typeArguments[3] = args.getArgumentValue(ARGUMENT_TYPETHREE);

		return typeArguments;
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {

		String urlArgString = (String) args.getArgumentValue(ARGUMENT_URL);
		String nodeArgString = (String) args.getArgumentValue(ARGUMENT_NODE);
		String expressionArgString = (String) args.getArgumentValue(ARGUMENT_EXPRESSION);

		return new StandardReadRegister(urlArgString, nodeArgString, expressionArgString, idVal);
	}

	private void executeReferencing(AutoTagBean input) throws AuthorOperationException {

		ArrayList<TagReferenceBookmarkBean> tagList = input.offsetsToTagList;
		String key = input.id;
		String selectedText = input.value;

		for (int i = tagList.size() - 1; i >= 0; i--) {

			TagReferenceBookmarkBean bookmark = tagList.get(i);

			String tag = "<rs " + XMLUtility.TEIXMLNS + " type='" + bookmark.type + "' key='" + key + "'/>";

			documentController.surroundInFragment(tag, bookmark.offset, bookmark.offset + selectedText.length() - 1);
		}
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
