package de.historischekommission_muenchen.oxydition.common.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen die Gliederungsebene
 * (abgebildet als n-Attribut) eines Dokumenteneintrag (abbgebildet als
 * <text>-Element) im Editionsgesamtdokument verändern können. Die Richtung und
 * Höhe der Verschiebung wird durch das Operationsargument festgelegt.
 * 
 * @author Maximilian Schrott
 * 
 */
public class FullEditionChangeDocumentRank extends OxyditionOperation {

	private static final String ARGUMENT_LEVELSHIFT = "Ebenendifferenz";

	private static final String levelShiftArgDescription = "Die Verschiebung in den Ebenen die durch die Operation ausgelöst werden soll.\n"
			+ "'+1' bedeutet z.B. das Dokument um eine Ebene nach unten verschoben wird,\n"
			+ "'-1', dass das Dokument um eine Ebene nach oben verschoben wird ";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_LEVELSHIFT, ArgumentDescriptor.TYPE_STRING, levelShiftArgDescription) };

	@Override
	public String getDescription() {
		return "Erlaubt es den Benutzer*innen die Gliederungsebene eine Dokuments im Gesamtdokument zu verändern. Richtung und Höhe der Verschiebung wird als Parameter übergeben";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		Object levelShiftArg = args.getArgumentValue("Ebenendifferenz");

		if (!OperationUtility.isValidArgument(levelShiftArg)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			AuthorElement changeNode = findValidChangeNode(surroundNode);

			if (changeNode != null) {
				String levelShiftVal = (String) levelShiftArg;
				AttrValue newLevelNumber = getNewLevelNumber(changeNode, levelShiftVal);
				documentController.setAttribute("n", newLevelNumber, changeNode);
			} else {
				displayInvalidCaretPositionMessage();
			}
		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (NumberFormatException e) {
			workspaceAccess.showErrorMessage(
					"Zahlenformatfehler: Entweder der Parameter dieser Operation oder das Ebenenattribut des gewählten Dokuments ist keine Zahl.\n"
							+ "Bitte wenden Sie sich an die technische Betreuung.");
		}

	}

	private AttrValue getNewLevelNumber(AuthorElement changeNode, String levelShiftVal) {

		String currentLevel = XMLUtility.getAttributeRawValue(changeNode, "n");
		String newLevel = calculateNewLevel(currentLevel, levelShiftVal);

		return new AttrValue(newLevel);
	}

	private String calculateNewLevel(String currentLevel, String levelShiftVal) throws NumberFormatException {

		int currentLevelInt = Integer.parseInt(currentLevel);
		int levelShiftInt = Integer.parseInt(levelShiftVal);
		int newLevelInt = currentLevelInt + levelShiftInt;

		// Ebene 1 ist die niedrigste Ebene
		if (newLevelInt < 1) {
			newLevelInt = 1;
		}

		return "" + newLevelInt;
	}

	private AuthorElement findValidChangeNode(AuthorElement surroundNode) {

		String surroundName = surroundNode.getLocalName();
		AuthorElement parentNode = XMLUtility.getNodeParentElement(surroundNode);

		if (surroundName == "TEI") {
			return null;
		} else if (surroundName == "text") {

			if (XMLUtility.elementDisplayNameEquals(parentNode, "group")
					&& (XMLUtility.elementAttributeEquals(parentNode, "n", "Dokumente"))) {
				return surroundNode;
			}
		}
		return findValidChangeNode(parentNode);
	}

	@Override
	protected Object getInvalidCaretPositionMessage() {
		return "Die Operation kann an der aktuellen Position des Textcursors nicht ausgeführt werden.\n"
				+ "Platzieren Sie diesen bitte innerhalb eines Dokumentenplatzhalters.";
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
