package de.historischekommission_muenchen.oxydition.common.operations.abstracts;

import java.io.File;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.node.AuthorDocument;

/**
 * Abstrakte Oberklasse für Operationen, die eine Liste von Dateien, als
 * String-Array benötigen. Die zentrale Methode getFileNames() findet alle
 * Dateien, im Ordner der aktuellen Datei. Alternativ kann über eine
 * Operationsvariable ein beliebiger Verzeichnispfad übergeben werden. Dann
 * findet die Operation alle Dateien, die sich in und unterhalb des
 * entsprechenden Verzeichnisses befinden. Wie die Operation auf die Auswahl der
 * Benutzer*innen reagiert, wird von den erbenden Klassen festgelegt
 * 
 * @author Maximilian Schrott
 *
 */
public abstract class FileListOperation extends OxyditionOperation {

	protected static String ARGUMENT_PARENTFOLDERPATH = "Übergeordneter Dateipfad:";

	protected static String parentFolderPathDescription = "Pfad zum Verzeichnis von dem aus alle untergeordneten Dateien gesucht werden sollen, z.B. ${pd}"
			+ "\n(optional, falls dieses Argument nich besetzt ist werden stattdessen alle Dateien angezeigt, die dem Verzeichnis der bearbeiteten Datei untergeordnet sind)";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] { new ArgumentDescriptor(
			ARGUMENT_PARENTFOLDERPATH, ArgumentDescriptor.TYPE_STRING, parentFolderPathDescription) };

	protected ArrayList<File> getFileList(ArgumentsMap args) {

		String parentFolderPathArgVal = (String) args.getArgumentValue(ARGUMENT_PARENTFOLDERPATH);

		ArrayList<File> fileList = new ArrayList<File>();

		/*
		 * Wenn Dateipfad-Parameter ausgefüllt ist, wird das entsprechende Verzeichnis
		 * als Ausgangsbasis benutzt und sämtliche diesem untergeordneten XML-Dateien
		 * ausgelesen; Wenn nicht werden nur die XML-Dateien direkt innerhalb des
		 * Verzeichnisse der aktuell bearbeiteten Datei ausgelesen
		 */

		if (OperationUtility.isValidArgument(parentFolderPathArgVal)) {
			fileList = getAllFilesBelowPath(parentFolderPathArgVal);
		} else {
			fileList = getFilesInDirectoryOfEditedDocument();
		}

		return fileList;
	}

	protected String[] getFileNames(ArrayList<File> fileList) {

		String[] fileNames = convertToFileNames(fileList);
		return fileNames;
	}

	protected String[] getFilePaths(ArrayList<File> fileList) {

		String[] filePaths = new String[fileList.size()];

		for (int i = 0; i < fileList.size(); i++) {
			filePaths[i] = fileList.get(i).getAbsolutePath();
		}

		return filePaths;

	}

	private ArrayList<File> getAllFilesBelowPath(String parentPathArgVal) {
		File parentDirectory = new File(parentPathArgVal);
		return getFilesBelowDirectory(parentDirectory);
	}

	private ArrayList<File> getFilesBelowDirectory(File directory) {

		ArrayList<File> fileList = new ArrayList<File>();
		fileList.addAll(getFilesInDirectory(directory));

		File[] childDirectories = directory.listFiles(File::isDirectory);

		for (int i = 0; i < childDirectories.length; i++) {
			File childDirectory = childDirectories[i];

			// versteckte Ordner z.B. ".PowerFolder" herausfiltern
			if (!childDirectory.getName().startsWith(".")) {
				// Rekursives Durchlaufen aller folgenden Unterordner
				fileList.addAll(getFilesBelowDirectory(childDirectory));
			}
		}

		return fileList;
	}

	private ArrayList<File> getFilesInDirectory(File directory) {

		File[] files = directory.listFiles();

		// Überspielen der Dateiliste erlaubt es erbenden Klassen diese zusätlich zu
		// filtern
		ArrayList<File> filteredFileList = filterFiles(files);

		return filteredFileList;
	}

	// Filtert Verzeichnisse und versteckte Dateien heraus. Erbende Klassen können
	// die Methode überschreiben um weitere/abweichende Filterkriterien hinzuzufügen
	protected ArrayList<File> filterFiles(File[] files) {
		ArrayList<File> fileredFileList = new ArrayList<File>();

		for (int i = 0; i < files.length; i++) {
			File file = files[i];

			if (!file.isDirectory() && !file.getName().startsWith(".")) {
				fileredFileList.add(file);
			}
		}

		return fileredFileList;
	}

	private ArrayList<File> getFilesInDirectoryOfEditedDocument() {
		String currentDocumentFilePath = getFilePath();
		File filesDirectory = new File(currentDocumentFilePath);
		ArrayList<File> fileList = getFilesInDirectory(filesDirectory);

		return fileList;
	}

	protected String getFilePath() {
		AuthorDocument document = documentController.getAuthorDocumentNode();
		String fullFileName = document.getSystemID();

		// .getSystemID gibt Dateinamen mit falsch escapten Leerzeichen aus - diese
		// müssen zurückumgewandelt werden
		fullFileName = fullFileName.replaceAll("%20", " ");
		return trimFilePath(fullFileName);
	}

	// Entfernt mit Regex Dateinamen und Endung aus dem Dateipfad
	protected String trimFilePath(String fullFileName) {

		Pattern fileNamePattern = Pattern.compile("(?:file:)(.*)(?=\\/)");
		Matcher fileNameMatcher = fileNamePattern.matcher(fullFileName);

		if (fileNameMatcher.find()) {
			return fileNameMatcher.group(1);
		}

		return "";
	}

	protected String[] convertToFileNames(ArrayList<File> fileList) {

		String[] fileNames = new String[fileList.size()];
		for (int i = 0; i < fileList.size(); i++) {
			File file = fileList.get(i);
			fileNames[i] = file.getName();
		}

		return fileNames;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

}
