package de.historischekommission_muenchen.oxydition.common.utilities;

import javax.swing.text.BadLocationException;

import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.content.TextContentIterator;
import ro.sync.ecss.extensions.api.content.TextContext;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;
import ro.sync.ecss.extensions.api.node.AuthorNode;

/**
 * Utility-Klasse, die statische Variablen und Funktionen zum Auslesen und
 * Bearbeiten von XML-Dateien anbietet
 * 
 * @author Maximilian Schrott
 * 
 */
public class XMLUtility {

	public static final String TEIXMLNS = "xmlns='http://www.tei-c.org/ns/1.0'";

	/**
	 * Überprüft, ob das übergebene XML-Element über das spezifizierte Attribut
	 * verfügt und ob dieses dem übergebenen String entspricht
	 *
	 * @param element        - das zu überprüfende XML-Element
	 * @param attributeName  - der Name des gesuchten Attributs
	 * @param attributeValue - der erwartete Wert des gesuchten Attributs
	 * @return "true" wenn das gesuchte Attribut vorhanden ist und den gesuchten
	 *         Wert hat - sonst "false"
	 */
	public static boolean elementAttributeEquals(AuthorElement element, String attributeName, String attributeValue) {

		AttrValue searchedAttribute = element.getAttribute(attributeName);

		if (searchedAttribute != null) {
			String searchAttributeValue = searchedAttribute.getRawValue();

			if (searchAttributeValue.equalsIgnoreCase(attributeValue)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Überprüft, ob das übergebene Element ein "type"-Attribut besitzt und ob es
	 * dem übergebenen String entspricht
	 *
	 * @param element        - das zu überprüfende XML-Element
	 * @param attributeValue - der erwartete Wert des "type"-Attributes
	 * @return "true" wenn der Wert das "type"-Attribut dem erwarteten Wert
	 *         entspricht - sonst "false"
	 */
	public static boolean elementTypeEquals(AuthorElement element, String attributeValue) {

		return elementAttributeEquals(element, "type", attributeValue);
	}

	/**
	 * Überprüft, ob der Name des übergebenen Elements dem übergebenen String
	 * entspricht
	 * 
	 * @param element     - das zu überprüfende XML-Element
	 * @param displayName - der erwartete Name des Elements
	 * @return "true" falls das Element den erwarteten lokalen Namen hat, sonst
	 *         "false"
	 */
	public static boolean elementDisplayNameEquals(AuthorElement element, String displayName) {

		String elementDisplayName = element.getDisplayName();
		if (elementDisplayName.equalsIgnoreCase(displayName)) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Überprüft ob der übergegebene Knoten ein list-Elemenzt ist, dessen
	 * "type"-Attribut den übergebenen String als Wert hat
	 * 
	 * @param element - das zu überprüfende XML-Element
	 * @param type    - der erwartete Wert des "type"-Attributs
	 * @return "true" falls das Element eine "list" ist und ein "type"-Attribut mit
	 *         dem erwarteten Wert hat, sonst "false"
	 */

	public static boolean elementIsListOfType(AuthorElement element, String type) {
		if (XMLUtility.elementDisplayNameEquals(element, "list")) {
			if (XMLUtility.elementTypeEquals(element, type)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * Ermittelt den Wert des "xml:id"-Attributs des Knotens, der das übergebene
	 * Dokumentenoffset umgibt
	 * 
	 * @param offset             - das zu überprüfende Offset im bearbeiteten
	 *                           Dokument
	 * @param documentController - die Oxygenschnittstelle zum bearbeiteten Dokument
	 * @return den Wert des "xml:id"-Attributs des Knotens an der Cursorposition;
	 *         Falls der Knoten keine xml:id hat, wird stattdessen einen leerer
	 *         String
	 * @throws BadLocationException
	 */
	public static String getSurroundNodeID(int offset, AuthorDocumentController documentController)
			throws BadLocationException {
		AuthorElement surroundNode = (AuthorElement) documentController.getNodeAtOffset(offset);

		return getAttributeRawValue(surroundNode, "xml:id");
	}

	/**
	 * Liest den String-Wert eines bestimmten Attributs aus dem übergebenen
	 * XML-Element aus
	 * 
	 * @param element  - das das Attribut enthaltende XML-Element
	 * @param attrName - der Name des gesuchten Attributs
	 * @return den Wert des gesuchten Attributs als String; Falls das übergebene
	 *         Element das gesuchte Attribut nicht besitzt, wird stattdessen einen
	 *         leeren String
	 */
	public static String getAttributeRawValue(AuthorElement element, String attrName) {

		AttrValue attrVal = element.getAttribute(attrName);

		if (attrVal != null) {
			return attrVal.getRawValue();
		} else {
			return "";
		}

	}

	/**
	 * Gibt das Elternelement (falls vorhanden) des übergebenen XML-Elements zurück
	 * 
	 * @param element - das zu überprüfende Element
	 * @return - das Elternelement des übergebeben Elements; Falls das übergebenene
	 *         Element der TEI-Wurzelelement ist, wird stattdessen das übergebene
	 *         Element selbst zurückgegeben
	 */
	public static AuthorElement getNodeParentElement(AuthorElement element) {

		AuthorNode parentNode = element.getParent();

		if (element.getDisplayName() != "TEI") {
			return (AuthorElement) parentNode;
		} else {
			return (AuthorElement) element;
		}
	}

	/**
	 * Gibt das erste Kindelement des übergebenden XML-Elements zurück, das den
	 * spezifizierten Namen hat (falls vorhanden)
	 * 
	 * @param element - der zu überprüfende Knoten
	 * @param name    - der lokale Name des gesuchten Kindknotens
	 * @return - den gefundenen Kindknoten; falls kein entsprechender Kindknoten
	 *         vorhanden ist, wird stattdessen NULL zurückgegeben
	 */
	public static AuthorElement getFirstChildByLocalName(AuthorElement element, String name) {

		AuthorElement[] children = element.getElementsByLocalName(name);

		if (children.length > 0) {
			return children[0];
		} else {
			return null;
		}
	}

	/**
	 * Gibt das erste Kindelement des übergebenden XML-Elements zurück, das den
	 * spezifizierten lokalen Namen und bei dem der Wert des "type"-Attributs dem
	 * übergebenen String entspricht (falls vorhanden)
	 * 
	 * @param element - der zu überprüfende Knoten
	 * @param name    - der lokale Name des gesuchten Kindelements
	 * @param type    - der Wert des "type"-Attributs des gesuchten Kindelements
	 * @return - den gefundenen Kindknoten; falls kein entsprechender Kindknoten
	 *         vorhanden ist, wird stattdessen NULL zurückgegeben
	 */
	public static AuthorElement getFirstChildByLocalNameAndType(AuthorElement element, String name, String type) {

		AuthorElement[] nameNodes = element.getElementsByLocalName(name);

		for (int i = 0; i < nameNodes.length; i++) {
			if (elementTypeEquals(nameNodes[i], type)) {
				return nameNodes[i];
			}
		}
		return null;
	}

	/**
	 * Sucht nach einem Vorfahren des übergebenen Elements, der den übergebenen
	 * Elementnamen hat. Die Suche wird abgebrochen, wenn die Operation auf das
	 * TEI-Wurzelelement trifft.
	 * 
	 * @param baseElement - Das Element von dem die Suche ausgehen soll
	 * @param localName   - Der erwartete Elementname des gesuchten Vorfahren
	 * @return den nähesten Vorfahrenknoten der den erwarteten namen hat; Falls kein
	 *         entsprechender Knoten vorhanden ist wird stattdessen NULL
	 *         zurückgegeben
	 */
	public static AuthorElement getAncestorElementWithLocalName(AuthorElement baseElement, String localName) {

		AuthorElement parentNode = (AuthorElement) baseElement.getParent();

		String parentName = parentNode.getLocalName();

		if (parentName == "TEI") {
			return null;
		} else if (parentName == localName) {
			return parentNode;
		} else {
			return getAncestorElementWithLocalName(parentNode, localName);
		}

	}

	/**
	 * Gibt den Textinhalt des ersten Kindelements des übergebenen XML-Elements
	 * zurück, das den spezifizierten lokalen Namen hat (falls vorhanden)
	 *
	 * @param parentElement - der zu überprüfende Knoten
	 * @param localName     - der lokale Name des gesuchten Kindelements
	 * @return - den Textinhalt des gefundenen Kindknotens; Falls kein
	 *         entsprechender Knoten vorhanden ist, wird stattdessen ein leerer
	 *         String zurückgegeben
	 * @throws BadLocationException
	 */
	public static String getChildText(AuthorElement parentElement, String localName) throws BadLocationException {
		AuthorElement childNode = XMLUtility.getFirstChildByLocalName(parentElement, localName);

		if (childNode != null) {
			return childNode.getTextContent();
		} else {
			return "";
		}
	}

	/**
	 * Gibt den Textinhalt des ersten Kindelements des übergebenen XML-Elements
	 * zurück, der den spezifizierten lokalen Namen hat und bei dem der Wert des
	 * "type"-Attributs dem übergebenen String entspricht (falls vorhanden)
	 * 
	 * @param parentElement - der zu überprüfende Knoten
	 * @param localName     - der lokale Names des gesuchten Kindelements
	 * @param type          - der erwartete Wert "type"-Attributs des gesuchten
	 *                      Kindknotens
	 * @return - den Textinhalt des gefundenen Kindkontens; Ist kein entsprechender
	 *         Knoten vorhanden, wird ein leerer String zurückgegeben
	 * @throws BadLocationException
	 */
	public static String getChildText(AuthorElement parentElement, String localName, String type)
			throws BadLocationException {
		AuthorElement childNode = XMLUtility.getFirstChildByLocalNameAndType(parentElement, localName, type);
		return childNode.getTextContent();
	}

	/**
	 * Ersetzt den Textinhalt des übergebenen XML-Elements mit dem übergebenen
	 * String
	 * 
	 * @param element            - das XML-Element, dessen Inhalt ersetzt werden
	 *                           soll
	 * @param replaceTextContent - der neue Textinhalt
	 * @param documentController - die Oxygenschnittstelle zum bearbeiteten Dokument
	 * @throws AuthorOperationException
	 */
	public static void replaceNodeTextContent(AuthorElement element, String replaceTextContent,
			AuthorDocumentController documentController) throws AuthorOperationException {

		int replaceOffset = deleteNodeTextContent(element, documentController);
		documentController.insertText(replaceOffset, replaceTextContent);
	}

	/**
	 * Entfernt den Inhalt des übergeben XML-Elements und gibt das Startoffset
	 * innerhalb des nun geleerten Elements zurück
	 * 
	 * @param element            - das Element dessen Inhalt gelöscht werden soll
	 * @param documentController - die Oxygenschnittstelle zum bearbeiteten Dokument
	 * @return - das Startoffset des Textinhalts, des geleerten Elements
	 */
	public static int deleteNodeTextContent(AuthorElement element, AuthorDocumentController documentController) {
		int nodeStartOffset = element.getStartOffset();
		int nodeEndOffset = element.getEndOffset();
		TextContentIterator iterator = documentController.getTextContentIterator(nodeStartOffset, nodeEndOffset);

		if (iterator.hasNext()) {
			TextContext context = iterator.next();
			int deleteStartOffset = context.getTextStartOffset();
			int deleteEndOffset = context.getTextEndOffset();

			documentController.delete(deleteStartOffset, deleteEndOffset);

			return deleteStartOffset;
		} else {
			return nodeEndOffset;
		}
	}

	/**
	 * Ersetzt das übergebene XML-Element durch das übergebene XML-Fragment
	 * 
	 * @param element                - das zu ersetzende Element
	 * @param replaceElementFragment - XML-Fragment, dass an der Position des
	 *                               ersetzten Knoten neu eingefügt werden soll
	 * @param documentController     - die Oxygenschnittstelle zum bearbeiteten
	 *                               Dokument
	 * @throws AuthorOperationException
	 */
	public static void replaceNode(AuthorElement element, String replaceElementFragment,
			AuthorDocumentController documentController) throws AuthorOperationException {

		int replaceOffset = deleteNodeContentNodes(element, documentController);
		documentController.insertXMLFragment(replaceElementFragment, replaceOffset);

	}

	private static int deleteNodeContentNodes(AuthorElement element, AuthorDocumentController documentController) {

		int nodeStartOffset = element.getStartOffset();
		int nodeEndOffset = element.getEndOffset();

		documentController.delete(nodeStartOffset, nodeEndOffset);

		return nodeStartOffset;
	}

	/**
	 * Prüft ob eine Position im Text direkt innerhalb eines XML-Element ist, das
	 * den übergebenen Namen besitzt
	 * 
	 * @param offset             - das zu überprüfende Offset im Text
	 * @param elementName        - der erwartete Name des umgebenden Elements
	 * @param documentController - die Oxygenschnittstelle zum bearbeiteten Dokument
	 * @return - true wenn sich das übergebene Offset innerhalb eines XML-Elements
	 *         mit dem erwarteten Namen befindet, sonst false
	 * @throws BadLocationException
	 */
	public static boolean offsetSurroundedByElement(int offset, String elementName,
			AuthorDocumentController documentController) throws BadLocationException {

		AuthorNode checkNode = documentController.getNodeAtOffset(offset);
		String checkNodeName = checkNode.getDisplayName();
		if (checkNodeName.equalsIgnoreCase(elementName)) {
			return true;
		}
		return false;
	}

	/**
	 * Prüft ob eine Position im Text direkt oder in tieferer Ebene innerhalb eine
	 * XML-Elements ist, das den übergebenen Namen besitzt
	 * 
	 * @param offset             - das zu überprüfende Offset im Text
	 * @param elementName        - der erwartete Name des umgebenden Elements
	 * @param documentController - die Oxygenschnittstelle zum bearbeiteten Dokument
	 * @return - true wenn das XML-Element, dass das Offset umgibt oder einer seiner
	 *         ::ancestor den gesuchten Namen besitzen, sonst false
	 * @throws BadLocationException
	 */
	public static boolean offsetInSelfOrDescendantOfElement(int offset, String elementName,
			AuthorDocumentController documentController) throws BadLocationException {

		if (offsetSurroundedByElement(offset, elementName, documentController)) {
			return true;
		} else {
			AuthorElement checkElement = (AuthorElement) documentController.getNodeAtOffset(offset);
			AuthorElement ancestorElement = getAncestorElementWithLocalName(checkElement, elementName);

			if (ancestorElement == null) {
				return false;
			} else {
				return true;
			}
		}

	}

	/**
	 * Ersetzt die in XML geschützten Zeichen innerhalb des übergeben Strings durch
	 * die sicheren Substitutes
	 * 
	 * @param text - der zu pürfende Text
	 * @return - den überarbeiteten Text; Ist der übergebene Text leer oder NULL
	 *         wird stattdesssen dieser zrückgegeben
	 */
	public static String escapeXML(String text) {

		if (text != null && text.length() > 0) {
			StringBuilder escapeBuilder = new StringBuilder();

			for (int i = 0; i < text.length(); i++) {
				String character = text.substring(i, i + 1);
				escapeBuilder.append(escapeCharacterXML(character));
			}

			return escapeBuilder.toString();

		} else {
			return text;
		}
	}

	private static String escapeCharacterXML(String character) {

		String[] charsToEscape = new String[] { "&", "<" /* , ">", "'", "\"" */ };
		String[] escapedchars = new String[] { "&amp;", "&lt;" /* , "&gt;", "&apos;", "&quot;" */ };

		String returnChar = character;
		for (int i = 0; i < charsToEscape.length; i++) {
			if (character.equalsIgnoreCase(charsToEscape[i])) {
				returnChar = escapedchars[i];
			}
		}
		return returnChar;
	}
}
