
package de.historischekommission_muenchen.oxydition.protedition.minrprot.operations;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorOperationException;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen die Grußworte am Ende
 * eines MinRProt-Ministerratprotokolls einfügen können. Die Grußworte bestehen
 * aus verschiedenen Teilelementen, deren Inhalt mittels der Operationsargumente
 * voreingestellt werden kann. So kann die Operation auf häufig auftretende
 * Personenkonstellationen konfiguriert werden. Werden die Argumente hingegen
 * leer gelassen kann auch eine leere Grußwortstruktur eingesetzt werden.
 * 
 * @author Maximilian Schrott
 * 
 */
public class InsertProtocolCloser extends OxyditionOperation {

	private static final String ARGUMENT_FUNCTION_ONE = "Funktion 1";
	private static final String ARGUMENT_NAME_ONE = "Name 1";
	private static final String ARGUMENT_OFFICE_ONE = "Amt 1";

	private static final String ARGUMENT_FUNCTION_TWO = "Funktion 2";
	private static final String ARGUMENT_NAME_TWO = "Name 2";
	private static final String ARGUMENT_OFFICE_TWO = "Amt 2";

	private static final String ARGUMENT_FUNCTION_THREE = "Funktion 3";
	private static final String ARGUMENT_NAME_THREE = "Name 3";
	private static final String ARGUMENT_OFFICE_THREE = "Amt 3";

	private static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_FUNCTION_ONE, ArgumentDescriptor.TYPE_STRING,
					"Die Funktion des ersten Unterzeichners, z.B.: Der Bayerische Ministerpräsident"),
			new ArgumentDescriptor(ARGUMENT_NAME_ONE, ArgumentDescriptor.TYPE_STRING,
					"Der Name des ersten Unterzeichners, z.B.: Horst Seehofer"),
			new ArgumentDescriptor(ARGUMENT_OFFICE_ONE, ArgumentDescriptor.TYPE_STRING,
					"Das Amt des ersten Unterzeicheners, z.B. Ministerialrat; Wenn nicht benötigt hier KEINAMT eingeben!"),
			new ArgumentDescriptor(ARGUMENT_FUNCTION_TWO, ArgumentDescriptor.TYPE_STRING,
					"Die Funktion des zweiten Unterzeichners"),
			new ArgumentDescriptor(ARGUMENT_NAME_TWO, ArgumentDescriptor.TYPE_STRING,
					"Der Name des zweiten Unterzeichners"),
			new ArgumentDescriptor(ARGUMENT_OFFICE_TWO, ArgumentDescriptor.TYPE_STRING,
					"Das Amt des zweiten Unterzeichners"),
			new ArgumentDescriptor(ARGUMENT_FUNCTION_THREE, ArgumentDescriptor.TYPE_STRING,
					"Die Funktion des dritten Unterzeichners"),
			new ArgumentDescriptor(ARGUMENT_NAME_THREE, ArgumentDescriptor.TYPE_STRING,
					"Der Name des dritten Unterzeichners"),
			new ArgumentDescriptor(ARGUMENT_OFFICE_THREE, ArgumentDescriptor.TYPE_STRING,
					"Das Amt der dritten Unterzeichners") };

	@Override
	public String getDescription() {

		return "Fügt die üblichen Grußworte (von Ministerpäsident,Leiter der Staatskanzlei und Generalsekretär des Ministerrats) ans "
				+ "Ende des Protokolls ein. Die verschiedenen Namen und Positionen können über die Operationsargumente vorausgefüllt werden";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args) throws AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		ProtocolCloserBean bean = new ProtocolCloserBean(args);

		String closer = constructProtocolCloserFragment(bean);

		// Fragment am Ende des Protokollabschnitts einsetzen
		String xpathLocation = "//div[@type='Protokoll']";

		documentController.insertXMLFragmentSchemaAware(closer, xpathLocation, AuthorConstants.POSITION_INSIDE_LAST);
	}

	private String constructProtocolCloserFragment(ProtocolCloserBean bean) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<closer " + XMLUtility.TEIXMLNS + ">");
		fragmentBuilder.append(constructSingleCloserFragment(bean.functionOne, bean.nameOne, bean.officeOne));
		fragmentBuilder.append(constructSingleCloserFragment(bean.functionTwo, bean.nameTwo, bean.officeTwo));
		fragmentBuilder.append(constructSingleCloserFragment(bean.functionThree, bean.nameThree, bean.officeThree));
		fragmentBuilder.append("</closer>");

		return fragmentBuilder.toString();
	}

	private String constructSingleCloserFragment(String function, String name, String office) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<seg type='Grusswort'><seg type='Funktion'>" + function + "</seg>");
		fragmentBuilder.append("<seg type='Unterschrift'> gez.: " + name + "</seg>");

		// Wurde "KEINAMT" in das Argumentfeld eingetragen, wird die
		// Amtsbezeichnung an dieser Stelle weggelassen
		if (!office.equalsIgnoreCase("KEINAMT")) {
			fragmentBuilder.append("<seg type='Amt'>" + office + "</seg>");
		}
		fragmentBuilder.append("</seg>");

		return fragmentBuilder.toString();
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		return ARGUMENTS;
	}

	private class ProtocolCloserBean {

		public String functionOne;
		public String functionTwo;
		public String functionThree;

		public String nameOne;
		public String nameTwo;
		public String nameThree;

		public String officeOne;
		public String officeTwo;
		public String officeThree;

		public ProtocolCloserBean(ArgumentsMap args) {
			fillBean(args);
		}

		private void fillBean(ArgumentsMap args) {
			functionOne = argumentToString(args.getArgumentValue(ARGUMENT_FUNCTION_ONE));
			nameOne = argumentToString(args.getArgumentValue(ARGUMENT_NAME_ONE));
			officeOne = argumentToString(args.getArgumentValue(ARGUMENT_OFFICE_ONE));

			functionTwo = argumentToString(args.getArgumentValue(ARGUMENT_FUNCTION_TWO));
			nameTwo = argumentToString(args.getArgumentValue(ARGUMENT_NAME_TWO));
			officeTwo = argumentToString(args.getArgumentValue(ARGUMENT_OFFICE_TWO));

			functionThree = argumentToString(args.getArgumentValue(ARGUMENT_FUNCTION_THREE));
			nameThree = argumentToString(args.getArgumentValue(ARGUMENT_NAME_THREE));
			officeThree = argumentToString(args.getArgumentValue(ARGUMENT_OFFICE_THREE));
		}

		private String argumentToString(Object arg) {

			String argString = (String) arg;

			if (argString == null) {
				return "";
			} else {
				return argString;
			}
		}

	}
}
