package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.GeoPointCoordinatesBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterGeoPointCoordinatesDialog;

import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorConstants;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Author-Oxygen-Operation mit der die Benutzer*innen die Geokoordinaten eins
 * Ortseintrages in der RDB bearbeiten können. Die Operation öffnet ein
 * Dialogfenster mit zwei Eingabefelder, in die die Benutzer*innen Längen- und
 * Breitengrad getrennt eintragen und das die Eingaben auf Korrektheit
 * überprüft.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterEditGeoPointCoordinates extends OxyditionOperation {

	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle können keine Geokoordinaten eingefügt werden. \n"
				+ "Führen Sie die Operation bitte aus, während der Textcursor sich innerhalb des Ortseintrags befinden,"
				+ " zu dem die Geokoordinaten hinzugefügt werden sollen.";
	}

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen Breiten- und Längengrad der Punktgeokoordinaten eine Ortes eingeben können."
				+ "Diese werden dann in das bzw. zusammen mit des entsprechende(n) Feld des Ortsregisterdatenbankeintrags"
				+ " an der Position des Textcursors eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			AuthorElement coordinatesNode = validateCaretPosition(surroundNode);

			if (coordinatesNode == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			RegisterGeoPointCoordinatesDialog dlg = new RegisterGeoPointCoordinatesDialog(oxygenFrame,
					"Geokoordinaten (Punkt) hinzufügen");
			GeoPointCoordinatesBean input = dlg.displayDialog();

			if (input.isFilled) {
				insertCorrdinates(coordinatesNode, input, documentController);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	/*
	 * Die Position des Textcursors ist nur gültig, wenn sie sich inner-/unterhalb
	 * eines <place>-Elements befindet, das wiederrum Kind eines
	 * <listPlace>-Elements ist. Zurückgegeben wird das niedrigste relevante, im
	 * Eintrag vorhandene Element (place, location[@type='Punkt'] oder geo)
	 */
	private AuthorElement validateCaretPosition(AuthorElement surroundnode) {

		AuthorElement placeNode = findPlaceNode(surroundnode);

		if (placeNode != null) {

			AuthorElement coordinatesNode = findCoordinatesNode(placeNode);
			if (coordinatesNode != null) {
				return coordinatesNode;
			}
		}

		return null;
	}

	private AuthorElement findPlaceNode(AuthorElement surroundnode) {

		AuthorElement placeNode = null;

		// Prüfen ob der Knoten an der Textposition ein <place>-Element oder das
		// Kind/Enkelkind eines solchen ist
		if (XMLUtility.elementDisplayNameEquals(surroundnode, "place")) {
			placeNode = surroundnode;
		} else if (XMLUtility.elementDisplayNameEquals(surroundnode, "geo")) {
			AuthorElement parentNode = XMLUtility.getNodeParentElement(surroundnode);
			AuthorElement grandParentNode = XMLUtility.getNodeParentElement(parentNode);

			if (XMLUtility.elementDisplayNameEquals(grandParentNode, "place")) {
				placeNode = grandParentNode;
			}
		} else {
			AuthorElement parentNode = XMLUtility.getNodeParentElement(surroundnode);

			if (XMLUtility.elementDisplayNameEquals(parentNode, "place")) {
				placeNode = parentNode;
			}
		}

		// Überprüfen ob sich der gefundene <place>-Knoten innerhalb eines
		// <listPlace>-Elements befindet
		if (placeNode != null) {
			AuthorElement listPlaceNode = XMLUtility.getNodeParentElement(placeNode);

			if (XMLUtility.elementDisplayNameEquals(listPlaceNode, "listPlace")) {
				return placeNode;
			} else {
				return null;
			}

		} else {
			return null;
		}

	}

	/*
	 * Gibt entweder den übergeben placeNode zurück, falls es keine Punkt-Location
	 * gibt Oder die Location falls diese kein <geo>-Kindelement besitzt oder das
	 * <geo>-Element, falls vorhanden
	 */
	private AuthorElement findCoordinatesNode(AuthorElement placeNode) {

		AuthorElement locationNode = XMLUtility.getFirstChildByLocalNameAndType(placeNode, "location", "Punkt");

		if (locationNode == null) {
			return placeNode;
		} else {
			AuthorElement geoNode = XMLUtility.getFirstChildByLocalName(locationNode, "geo");

			if (geoNode == null) {
				return locationNode;
			} else {
				return geoNode;
			}
		}
	}

	private void insertCorrdinates(AuthorElement coordinatesNode, GeoPointCoordinatesBean input,
			AuthorDocumentController documentController) throws AuthorOperationException {

		String coordinateFragment = constructCoordinatesFragment(input, coordinatesNode);
		AuthorElement insertionNode;

		// Falls ein <geo>-Element übergeben wurde, wird dieses gelöscht und durch das
		// neu angelegte ersetzt
		if (XMLUtility.elementDisplayNameEquals(coordinatesNode, "geo")) {
			insertionNode = XMLUtility.getNodeParentElement(coordinatesNode);
			documentController.deleteNode(coordinatesNode);
		} else {
			insertionNode = coordinatesNode;
		}

		documentController.insertXMLFragment(coordinateFragment, insertionNode, AuthorConstants.POSITION_INSIDE_LAST);

	}

	private String constructCoordinatesFragment(GeoPointCoordinatesBean input, AuthorElement coordinatesNode) {

		StringBuilder fragmentBuilder = new StringBuilder();

		if (coordinatesNode.getDisplayName() == "place") {
			fragmentBuilder.append("<location " + XMLUtility.TEIXMLNS + " type='Punkt'>");
		}

		fragmentBuilder.append("<geo " + XMLUtility.TEIXMLNS + ">");
		fragmentBuilder.append(input.latitude + "," + input.longitude);
		fragmentBuilder.append("</geo>");

		if (coordinatesNode.getDisplayName() == "place") {
			fragmentBuilder.append("</location>");
		}

		return fragmentBuilder.toString();

	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

}
