package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.JFrame;
import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.LiteratureTitleContentBean;
import de.historischekommission_muenchen.oxydition.register.beans.LiteratureTitlePartBean;
import de.historischekommission_muenchen.oxydition.register.beans.NewLiteraturTitleBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterLiteratureTitleDialog;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation zum Bearbeiten bestehender Literatur-Einträge in der
 * RDB. Die Operation öffnet das identische Dialogfenster wie beim Einfügen
 * neuer Literatur-Einträge, wandelt die bestehenden Datenfelder zurück in das
 * Eingabeformat um und fügt sie in die entsprechenden Textfelder ein. Die
 * Änderungen der Benutzer*innen werden dann in die RDB zurückgeschrieben.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterEditLiteratureTitle extends RegisterNewLiteratureTitle {

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement entryNode = findLiteratureEntryNode(surroundNode);

			if (entryNode == null) {

				displayInvalidCaretPositionMessage();

			} else {

				LiteratureTitleContentBean currentEntryContent = getLiteratureEntryContent(entryNode);

				NewLiteraturTitleBean input = displayLiteratureTitleDialog(oxygenFrame, currentEntryContent);

				if (input.isFilled) {
					updateLiteratureTitle(input, entryNode);
				}
			}
		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}

	}

	private AuthorElement findLiteratureEntryNode(AuthorElement node) {

		if (node.getLocalName() == "bibl") {
			return node;
		} else {
			return XMLUtility.getAncestorElementWithLocalName(node, "bibl");
		}
	}

	private LiteratureTitleContentBean getLiteratureEntryContent(AuthorElement entryNode) throws BadLocationException {

		LiteratureTitleContentBean content = new LiteratureTitleContentBean();

		content.fullTitle = getFullTitleText(entryNode);
		content.shortTitle = getShortTitleText(entryNode);
		content.date = getDateText(entryNode);

		return content;
	}

	private String getFullTitleText(AuthorElement entryNode) throws BadLocationException {

		AuthorElement[] titleSegments = getTitleSegments(entryNode, "Volltitel");
		String titleText = constructTitleText(titleSegments);

		return titleText;
	}

	private String getShortTitleText(AuthorElement entryNode) throws BadLocationException {

		AuthorElement[] titleSegments = getTitleSegments(entryNode, "Kurztitel");
		String titleText = constructTitleText(titleSegments);

		return titleText;
	}

	private AuthorElement[] getTitleSegments(AuthorElement entryNode, String titleType) {

		AuthorElement titleNode = XMLUtility.getFirstChildByLocalNameAndType(entryNode, "title", titleType);

		return titleNode.getElementsByLocalName("seg");
	}

	private String constructTitleText(AuthorElement[] titleSegments) throws BadLocationException {

		StringBuilder titleTextBuilder = new StringBuilder();

		for (int i = 0; i < titleSegments.length; i++) {

			AuthorElement segment = titleSegments[i];

			titleTextBuilder.append(markHighlightedSegment(segment));

		}

		return titleTextBuilder.toString();
	}

	private Object markHighlightedSegment(AuthorElement segment) throws BadLocationException {

		String segmentType = XMLUtility.getAttributeRawValue(segment, "type");
		String segmentText = segment.getTextContent();

		if (segmentType.equalsIgnoreCase("Kennabschnitt")) {
			return "<" + segmentText + ">";
		} else {
			return segmentText;
		}
	}

	private String getDateText(AuthorElement entryNode) throws BadLocationException {

		AuthorElement dateNode = XMLUtility.getFirstChildByLocalName(entryNode, "date");

		return dateNode.getTextContent();
	}

	private NewLiteraturTitleBean displayLiteratureTitleDialog(JFrame oxygenFrame,
			LiteratureTitleContentBean currentEntryContent) {

		NewLiteraturTitleBean input = new NewLiteraturTitleBean();

		RegisterLiteratureTitleDialog dlg;
		dlg = new RegisterLiteratureTitleDialog(oxygenFrame, "Neuen Literaturtitel ins Register einfügen");
		dlg.presetContent(currentEntryContent);
		input = dlg.displayDialog();
		return input;
	}

	private void updateLiteratureTitle(NewLiteraturTitleBean input, AuthorElement entryNode)
			throws AuthorOperationException, BadLocationException {
		updateFullTitle(input, entryNode);
		updateShortTitle(input, entryNode);
		updateDate(input, entryNode);
	}

	private void updateFullTitle(NewLiteraturTitleBean input, AuthorElement entryNode)
			throws AuthorOperationException, BadLocationException {

		AuthorElement fullTitleNode = XMLUtility.getFirstChildByLocalNameAndType(entryNode, "title", "Volltitel");
		String updatedFullTitle = constructUpdatedFullTitleFragment(input.fullTitle);

		XMLUtility.replaceNode(fullTitleNode, updatedFullTitle, documentController);
	}

	private void updateShortTitle(NewLiteraturTitleBean input, AuthorElement entryNode)
			throws AuthorOperationException {

		AuthorElement shortTitleNode = XMLUtility.getFirstChildByLocalNameAndType(entryNode, "title", "Kurztitel");
		String updatedShortTitle = constructUpdatedShortTitleFragment(input.shortTitle);

		XMLUtility.replaceNode(shortTitleNode, updatedShortTitle, documentController);
	}

	private String constructUpdatedFullTitleFragment(LiteratureTitlePartBean[] fullTitle) {

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + " type='Volltitel'>");
		fragmentBuilder.append(constructFullTitleFragment(fullTitle));
		fragmentBuilder.append("</title>");

		return fragmentBuilder.toString();
	}

	private String constructUpdatedShortTitleFragment(LiteratureTitlePartBean[] shortTitle) {
		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append("<title " + XMLUtility.TEIXMLNS + " type='Kurztitel'>");
		fragmentBuilder.append(constructShortTitleFragment(shortTitle));
		fragmentBuilder.append("</title>");

		return fragmentBuilder.toString();
	}

	private void updateDate(NewLiteraturTitleBean input, AuthorElement entryNode) throws AuthorOperationException {

		AuthorElement dateNode = XMLUtility.getFirstChildByLocalName(entryNode, "date");
		XMLUtility.replaceNodeTextContent(dateNode, input.date, documentController);
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "Keine Literaturtitel zum bearbeiten gewählt!\n"
				+ "Bitte positionieren Sie den Textcursor innerhalb eines Literatureintrags, um die Operation auszuführen.";
	}

}
