package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen eine Personenkurzreferenz
 * bzw. einen Biogrammrumpf für einen Personen-RDB-Eintrag erzeugen können. Die
 * Operation liest das <persName @Lesename>-, <note @type="Qualifikator">-,
 * <birth/>- und <death/>-Element ein und formt daraus ein Textfragment, dass
 * dann in das <note @type="Kurzreferenz"> bzw. <note @type="Biogramm>-Element
 * des Eintrag eingefügt wird.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterInsertPersonQuickReference extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Liest den Inhalt der Datenfelder \"Lesename\", \"Qualifikator\", \"Geburtsdatum\" und \"Todesdatum\" aus und "
				+ "fügt sie in das Feld \"Kurzreferenz\" bzw. \"Biogramm\" ein "
				+ "(der bisherige Inhalt des Feldes wird dabei gelöscht) ";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);
		AuthorElement surroundNode;
		;
		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			AuthorElement personNode = XMLUtility.getAncestorElementWithLocalName(surroundNode, "person");

			if (personNode != null) {
				insertQuickReferenceFragment(personNode);

			} else {
				displayInvalidCaretPositionMessage();
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	protected void insertQuickReferenceFragment(AuthorElement personNode)
			throws BadLocationException, AuthorOperationException {
		String quickReferenceFragment = constructQuickReferenceFragment(personNode);
		AuthorElement referenceNode = getReferenceNode(personNode);
		XMLUtility.replaceNodeTextContent(referenceNode, quickReferenceFragment, documentController);
	}

	protected String constructQuickReferenceFragment(AuthorElement personNode) throws BadLocationException {

		String nameText = XMLUtility.getChildText(personNode, "persName", "Lesename");
		nameText = XMLUtility.escapeXML(nameText);
		String qualificatorText = XMLUtility.getChildText(personNode, "note", "Qualifikator");
		qualificatorText = XMLUtility.escapeXML(qualificatorText);
		String birthText = XMLUtility.getChildText(personNode, "birth");
		birthText = XMLUtility.escapeXML(birthText);
		String deathText = XMLUtility.getChildText(personNode, "death");
		deathText = XMLUtility.escapeXML(deathText);

		StringBuilder fragmentBuilder = new StringBuilder();
		fragmentBuilder.append(nameText);
		if (qualificatorText.length() > 0) {
			fragmentBuilder.append(", " + qualificatorText);
		}
		if (birthText.length() > 0 || deathText.length() > 0) {
			fragmentBuilder.append(" (" + birthText + "-" + deathText + ")");
		}

		return fragmentBuilder.toString();
	}

	private AuthorElement getReferenceNode(AuthorElement persNode) {

		AuthorElement referenceNode = XMLUtility.getFirstChildByLocalNameAndType(persNode, "note", "Kurzreferenz");

		if (referenceNode == null) {
			referenceNode = XMLUtility.getFirstChildByLocalNameAndType(persNode, "note", "Biogramm");
		}

		return referenceNode;
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return null;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann keine Personenkurzreferenz eingegeben werden";
	}

}
