package de.historischekommission_muenchen.oxydition.register.operations;

import java.io.FileNotFoundException;
import java.io.IOException;

import javax.swing.text.BadLocationException;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPathExpressionException;

import org.xml.sax.SAXException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.ReadRegisterOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.KeywordReadRegister;
import de.historischekommission_muenchen.oxydition.common.utilities.OperationUtility;
import de.historischekommission_muenchen.oxydition.common.utilities.ReadRegisterBasic;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.RegisterKeywordBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterKeywordNestedDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AttrValue;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Oxygen-Author-Operation mit der die Benutzer*innen einen neuen
 * Sachschlagworteintrag in einer geschachtelten Sachschlagwort-RDB-Liste
 * einfügen können. Bietet die Möglichkeit dem Eintrag, zusätzlich zum Begriff,
 * auch Querverweise auf andere Sachschlagwort-Einträge hinzuzufügen.
 * 
 * @author Maximilian Schrott
 *
 */

public class RegisterNewKeywordNested extends ReadRegisterOperation {

	protected RegisterKeywordBean input;

	protected static final ArgumentDescriptor[] ARGUMENTS = new ArgumentDescriptor[] {

			new ArgumentDescriptor(ARGUMENT_URL, ArgumentDescriptor.TYPE_STRING, urlArgDescription), };

	@Override
	public String getDescription() {
		return "Öffnet ein Dialogfenster, in dem die Benutzer*innen ein neues Schlagwort für das Register eintragen und einfügen können."
				+ "Zusätlich ist es möglich das neue Schlagwort mit schon bestehenden Schlagwörtern zu verknüpfen. "
				+ "Das neue Schlagwort wird an der Cursorposition in das aktuelle Dokument eingefügt";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		AuthorElement surroundNode;

		if (!validateUrlArgument(args)) {
			displayInvalidArgumentsMessage();
			return;
		}

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);

			AuthorElement listOrItem = validateCursorPosition(surroundNode);

			if (listOrItem == null) {
				displayInvalidCaretPositionMessage();
				return;
			}

			ReadRegisterBasic readRegister = getRegisterEntries(args);
			String[] eintraege = readRegister.getEintrag();
			String[] ids = readRegister.getID();

			RegisterKeywordNestedDialog dlg = new RegisterKeywordNestedDialog(oxygenFrame,
					"Neues Sachschlagwort eingeben", eintraege, ids);
			input = dlg.displayDialog();

			if (input.isFilled) {
				insertNewKeywordFragment(input, listOrItem);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		} catch (FileNotFoundException e) {
			displayRegisterNotFoundMessage();
		} catch (XPathExpressionException | ParserConfigurationException | IOException | SAXException e) {
			OperationUtility.handleGenericException(oxygenFrame, e);
		}
	}

	/** Prüft ob Cursor sich an gültiger Position befindet */
	private AuthorElement validateCursorPosition(AuthorElement surroundNode) {
		// Cursor in..

		// Schlagwortliste - gültige Position
		if (XMLUtility.elementIsListOfType(surroundNode, "Sachschlagwörter")) {
			return surroundNode;
		}

		// In <item>-Element - gültige Position, falls das <item> nicht bereits
		// eine Sachschlagwortsliste enthält
		else if (XMLUtility.elementDisplayNameEquals(surroundNode, "item")) {
			if (RegisterUtility.elementContainsListOfType(surroundNode, "Sachschlagwörter")) {
				return null;
			} else {
				return surroundNode;
			}

		}

		// In <term>- oder <ref>-Element - umgebendes <item>-Element überprüfen
		// - gültig
		// wenn dieses nicht bereits eine Schlagwortliste enthält
		else if (XMLUtility.elementDisplayNameEquals(surroundNode, "term")
				|| XMLUtility.elementDisplayNameEquals(surroundNode, "ref")) {
			AuthorElement itemElement = (AuthorElement) surroundNode.getParent();

			if (RegisterUtility.elementContainsListOfType(itemElement, "Sachschlagwörter")) {
				return null;
			} else {
				return itemElement;
			}
		} else {
			return null;
		}
	}

	protected void insertNewKeywordFragment(RegisterKeywordBean input2, AuthorElement listOrItem)
			throws AuthorOperationException {
		boolean isItem = XMLUtility.elementDisplayNameEquals(listOrItem, "item");

		boolean newKeywordListNeeded = isItem
				&& !RegisterUtility.elementContainsListOfType(listOrItem, "Sachschlagwörter");

		if (newKeywordListNeeded) {
			insertKeywordListIntoItem(documentController, listOrItem);
			caretPosition = editorAccess.getCaretOffset();
		}

		String newKeywordNestedFragment = constructKeyWordNestedFragment();
		documentController.insertXMLFragment(newKeywordNestedFragment, caretPosition);
	}

	protected String constructKeyWordNestedFragment() {

		String newKeywordId = RegisterUtility.generateID(input.keywordTerm, false, 20);
		String keywordTerm = XMLUtility.escapeXML(input.keywordTerm);

		StringBuilder newKeywordBuilder = new StringBuilder();
		newKeywordBuilder.append("<item " + XMLUtility.TEIXMLNS + " xml:id='" + newKeywordId + "'>");
		newKeywordBuilder.append("<term " + XMLUtility.TEIXMLNS + ">" + keywordTerm + "</term>");

		newKeywordBuilder.append(constructCrossReferenceFragment());
		newKeywordBuilder.append("</item>");

		return newKeywordBuilder.toString();

	}

	protected String constructCrossReferenceFragment() {

		StringBuilder fragmentBuilder = new StringBuilder();

		for (int i = 0; i < input.getKeywordCrossReferenceSize(); i++) {

			fragmentBuilder.append("<ref " + XMLUtility.TEIXMLNS + " type='Querverweis' ");
			fragmentBuilder.append("target='" + input.getCrossReferenceId(i) + "'/>");
		}

		return fragmentBuilder.toString();
	}

	private void insertKeywordListIntoItem(AuthorDocumentController documentController, AuthorElement surroundNode)
			throws AuthorOperationException {

		int newListDegree = getDegreeOfCurrentList(surroundNode);

		String keywordListFragment = "<list " + XMLUtility.TEIXMLNS + " type='Sachschlagwörter' n='" + newListDegree
				+ "'>" + "</list>";

		int internalEndOffset = surroundNode.getEndOffset();
		documentController.insertXMLFragment(keywordListFragment, internalEndOffset);
	}

	private int getDegreeOfCurrentList(AuthorElement surroundNode) {

		AuthorElement surroundList = (AuthorElement) surroundNode.getParent();

		AttrValue nValue = surroundList.getAttribute("n");

		int nInteger = Integer.parseInt(nValue.getRawValue());

		return nInteger + 1;
	}

	@Override
	protected ReadRegisterBasic getRegisterEntries(ArgumentsMap args)
			throws XPathExpressionException, ParserConfigurationException, IOException, SAXException {
		String urlArgVal = (String) args.getArgumentValue(ARGUMENT_URL);

		return new KeywordReadRegister(urlArgVal);
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argumente
		return ARGUMENTS;
	}

	@Override
	protected String getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neues Sachschlagwort eingefügt werden.\n"
				+ "Fügen Sie den Eintrag bitte innerhalb einer bestehenden Schlagwortliste oder innerhalb eines Sachschlagworts, "
				+ "dass noch keine Untersachschlagwortsliste hat ein.\n";
	}

}
