package de.historischekommission_muenchen.oxydition.register.operations;

import javax.swing.text.BadLocationException;

import de.historischekommission_muenchen.oxydition.common.operations.abstracts.OxyditionOperation;
import de.historischekommission_muenchen.oxydition.common.utilities.XMLUtility;
import de.historischekommission_muenchen.oxydition.register.beans.CollectionHierarchyBean;
import de.historischekommission_muenchen.oxydition.register.beans.NewUnpublishedSourceBean;
import de.historischekommission_muenchen.oxydition.register.beans.SourceCollectionBean;
import de.historischekommission_muenchen.oxydition.register.dialogs.RegisterNewUnpublishedSourceDialog;
import de.historischekommission_muenchen.oxydition.register.utilities.RegisterUtility;
import ro.sync.ecss.extensions.api.ArgumentDescriptor;
import ro.sync.ecss.extensions.api.ArgumentsMap;
import ro.sync.ecss.extensions.api.AuthorAccess;
import ro.sync.ecss.extensions.api.AuthorDocumentController;
import ro.sync.ecss.extensions.api.AuthorOperationException;
import ro.sync.ecss.extensions.api.node.AuthorElement;

/**
 * Operation mit der die Benutzer*innen einen neuen Quellen-Eintrag in ein
 * Archiv oder einen Bestand in der RDB für ungedruckte Quellen einfügen können.
 * 
 * @author Maximilian Schrott
 *
 */
public class RegisterNewUnpublishedSource extends OxyditionOperation {

	@Override
	public String getDescription() {
		return "Dialog zum Einfügen eines neuen Eintrags für eine ungedruckte Quelle in des Register";
	}

	@Override
	public void doOperation(AuthorAccess authorAccess, ArgumentsMap args)
			throws IllegalArgumentException, AuthorOperationException {

		initializeOxygenInterfaceVariables(authorAccess);

		boolean caretWithinSourceList = false;

		AuthorElement surroundNode;

		try {
			surroundNode = (AuthorElement) documentController.getNodeAtOffset(caretPosition);
			caretWithinSourceList = isSourceList(surroundNode);
			CollectionHierarchyBean hierarchybean = findCollectionHierarchy(surroundNode, caretWithinSourceList);

			if (!hierarchybean.validHierarchy) {
				displayInvalidCaretPositionMessage();
				return;
			}

			RegisterNewUnpublishedSourceDialog dlg = new RegisterNewUnpublishedSourceDialog(oxygenFrame, hierarchybean);
			NewUnpublishedSourceBean input = dlg.displayDialog();

			if (input.isFilled) {
				if (!caretWithinSourceList) {
					insertSourceListIntoCollection(documentController, caretPosition);
					caretPosition = editorAccess.getCaretOffset();
				}
				insertNewUnpublishedSource(input, hierarchybean);
			}

		} catch (BadLocationException e) {
			displayBadLocationMessage(e);
		}
	}

	@Override
	public ArgumentDescriptor[] getArguments() {
		// Keine Argmeunte
		return null;
	}

	private void insertSourceListIntoCollection(AuthorDocumentController documentController, int caretPosition)
			throws AuthorOperationException {

		String sourceListFragment = "<list xmlns='http://www.tei-c.org/ns/1.0' type='Quellen'></list>";

		documentController.insertXMLFragment(sourceListFragment, caretPosition);
	}

	private boolean isSourceList(AuthorElement node) {

		return (XMLUtility.elementDisplayNameEquals(node, "list") && XMLUtility.elementTypeEquals(node, "Quellen"));

	}

	private CollectionHierarchyBean findCollectionHierarchy(AuthorElement surroundNode, boolean caretWithinSoureList)
			throws BadLocationException {

		CollectionHierarchyBean hierarchyBean = new CollectionHierarchyBean();
		AuthorElement surroundNodeParent = (AuthorElement) surroundNode.getParent();

		if (caretWithinSoureList) {

			AuthorElement surroundNodeGrandparent = (AuthorElement) surroundNodeParent.getParent();

			if (XMLUtility.elementIsListOfType(surroundNodeGrandparent, "Archive")) {
				hierarchyBean = hierarchyBeanFromArchive(surroundNodeParent);
			} else if (XMLUtility.elementIsListOfType(surroundNodeGrandparent, "Bestände")) {
				hierarchyBean = hierarchyBeanFromStock(surroundNodeParent);
			}

		} else {

			if (!RegisterUtility.elementContainsListOfType(surroundNode, "Quellen")) {

				if (XMLUtility.elementIsListOfType(surroundNodeParent, "Archive")) {
					hierarchyBean = hierarchyBeanFromArchive(surroundNode);
				} else if (XMLUtility.elementIsListOfType(surroundNodeParent, "Bestände")) {
					hierarchyBean = hierarchyBeanFromStock(surroundNode);
				}
			}
		}

		return hierarchyBean;
	}

	private CollectionHierarchyBean hierarchyBeanFromArchive(AuthorElement archiveNode) throws BadLocationException {

		CollectionHierarchyBean hierarchyBean = new CollectionHierarchyBean();
		hierarchyBean.archiveBean = getCollectionNodeData(archiveNode);
		hierarchyBean.validHierarchy = true;

		return hierarchyBean;
	}

	private CollectionHierarchyBean hierarchyBeanFromStock(AuthorElement stockNode) throws BadLocationException {
		CollectionHierarchyBean hierarchyBean = new CollectionHierarchyBean();

		hierarchyBean.stockBean = getCollectionNodeData(stockNode);
		AuthorElement stockList = (AuthorElement) stockNode.getParent();
		AuthorElement archiveNode = (AuthorElement) stockList.getParent();
		hierarchyBean.archiveBean = getCollectionNodeData(archiveNode);
		hierarchyBean.validHierarchy = true;

		return hierarchyBean;
	}

	private SourceCollectionBean getCollectionNodeData(AuthorElement collectionNode) throws BadLocationException {

		String collectionID = collectionNode.getAttribute("xml:id").getRawValue();
		AuthorElement[] archiveNames = collectionNode.getElementsByLocalName("name");

		SourceCollectionBean collectionBean = new SourceCollectionBean(collectionID, archiveNames);

		return collectionBean;
	}

	private void insertNewUnpublishedSource(NewUnpublishedSourceBean input, CollectionHierarchyBean hierarchybean)
			throws AuthorOperationException {
		String newUnpublishedSourceFragment = constructNewUnpublishedSourceXMLFragment(input, hierarchybean);
		documentController.insertXMLFragment(newUnpublishedSourceFragment, caretPosition);
	}

	private String constructNewUnpublishedSourceXMLFragment(NewUnpublishedSourceBean input,
			CollectionHierarchyBean hierarchyBean) {

		String parentCollectionId = getParentCollectionId(hierarchyBean);
		String sourceId = generateSourceId(input.title, parentCollectionId);
		String title = XMLUtility.escapeXML(input.title);
		String desc = XMLUtility.escapeXML(input.desc);

		String fragment = "<item xmlns='http://www.tei-c.org/ns/1.0' xml:id='" + sourceId + "'>";
		fragment += "<title xmlns='http://www.tei-c.org/ns/1.0'>" + title + "</title>";
		if (!input.desc.isEmpty()) {
			fragment += "<desc xmlns='http://www.tei-c.org/ns/1.0'>" + desc + "</desc>";
		}
		fragment += "</item>";

		return fragment;
	}

	private String getParentCollectionId(CollectionHierarchyBean hierarchyBean) {

		if (hierarchyBean.stockBean != null) {
			return hierarchyBean.stockBean.id;
		} else {
			return hierarchyBean.archiveBean.id;
		}
	}

	// Id des neuen Quelleneintrags erzeugen aus der ID des übergeordneten
	// Archivs oder Bestandes (ohne Zufallszahl) und dem Titel der Quelle
	private String generateSourceId(String sourceTitle, String parentCollectionId) {

		String[] parentCollectionIdSplit = parentCollectionId.split("_");
		String parentCollectionIdBase = parentCollectionIdSplit[0];

		String sourceId = parentCollectionIdBase + RegisterUtility.generateID(sourceTitle, true, 20);

		return sourceId;
	}

	@Override
	protected Object getInvalidCaretPositionMessage() {
		return "An dieser Stelle kann kein neuer Eintrag für eine ungedruckte Quelle eingefügt werden.\n"
				+ "Fügen Sie den Eintrag bitte innerhalb einer Quellenliste innerhalb eins Bestands oder Archivs ein oder in einem noch leeren Bestand oder Archiv ein.\n"
				+ "Achten Sie bitte darauf, dass sich der TextCursor nicht innerhalb eines anderen Quelleneintrags befindet";
	}

}
